// bitovi.h - Klasa nizova bitova.

#ifndef _bitovi_h_
#define _bitovi_h_

#include <iostream>
using namespace std;

#define DUZ ((D+7)>>3)       // Broj bajtova u nizu.
#define MASKA (~(~0<<(D&7))) // Maska za ciscenje nekoriscenih levih
                             //   bitova poslednjeg elementa niza.

typedef unsigned char Uchar; // Skracenice za neoznacene celobrojne tipove.
typedef unsigned long Ulong;

class GPoz {};         // KLASA ZA GRESKU: Nedovoljena pozicija bita.

template <int D>
  class Bitovi {
    class Bit {          // KLASA ZA PREDSTAVLJANJE ODABRANOG BITA U NIZU.
      Uchar* bajt;       // Bajt u kome se nalazi bit.
      unsigned maska;    // Maska za izdvajanje bita iz bajta.
    public:
      Bit (Uchar* niz, unsigned poz)     // Konstruktor.
        { bajt = niz + (poz >> 3); maska = 1 << (poz & 7); }
      operator bool () const {           // Dohvatanje vrednosti bita.
        return (*bajt & maska) != 0;
      }
      Bit& operator= (bool b) {          // Postavljanje vrednosti bita.
        if (b) *bajt |= maska; else *bajt &= ~maska;
        return *this;
      }
      Bit& operator= (const Bit& bit) {  //   (za slucaj: a[i]=b[j])
        return *this = bool (bit);
      }
    }; // class Bit
                                         // CLANOVI KLASE NIZA BITOVA:
    Uchar niz[DUZ];                      // Niz za uskladistavanje bitova.
  public:
    Bitovi (Ulong k=0);                  // Konstruktor konverzije.
    Bit operator[] (int i) {             // Pristup odabranom bitu:
      if (i<0 || i>=D) throw GPoz ();    //   u promenljivom nizu,
      return Bit (niz, i);
    }
    const Bit operator[] (int i) const { //   u nepromenljivom nizu.
      if (i<0 || i>=D) throw GPoz ();
      return Bit (const_cast<Uchar*>(niz), i);
    }
    typedef const Bitovi CB; // Skracenica za nepromenljive nizove.
    Bitovi  operator~ () const;          // ~b1
    Bitovi& operator&= (CB& b2);         // b1 &= b2
    Bitovi& operator|= (CB& b2);         // b1 |= b2
    Bitovi& operator^= (CB& b2);         // b1 ^= b2
    Bitovi& operator>>=(int k);          // b1 >>= k
    Bitovi& operator<<=(int k);          // b1 <<= k
    bool    operator== (CB& b2) const;   // b1 == b2
    bool    operator<  (CB& b2) const;   // b1 <  b2
    bool    operator>  (CB& b2) const;   // b1 >  b2
  }; // class Bitovi

#define CB const Bitovi<D> // Skracenica za nepromenljive nizove.

template <int D>
  Bitovi<D>::Bitovi (Ulong k) {         // Konstruktor konverzije
    for (int i=0, j=-1, m; i<D; i++) {  //   (koristi desnih D bitova).
      if ((i & 7) == 0) { m = 1; j++; } else  m <<= 1;
      if (k & 1) niz[j] |= m; else niz[j] &= ~m;
      k >>= 1;
    }
    niz[DUZ-1] &= MASKA;
  }

template <int D>
  Bitovi<D> Bitovi<D>::operator~ () const {        // ~b1
    Bitovi<D> b;
    for (int i=0; i<DUZ; i++) b.niz[i] = ~ niz[i];
    b.niz[DUZ-1] &= MASKA;
    return b;
  }

template <int D>
  Bitovi<D>& Bitovi<D>::operator&= (CB& b2) {      // b1 &= b2
    for (int i=0; i<DUZ; i++) niz[i] &= b2.niz[i];
    return *this;
  }
  
template <int D>
  Bitovi<D>& Bitovi<D>::operator|= (CB& b2) {      // b1 |= b2
    for (int i=0; i<DUZ; i++) niz[i] |= b2.niz[i];
    return *this;
  }

template <int D>
  Bitovi<D>& Bitovi<D>::operator^= (CB& b2) {      // b1 ^= b2
    for (int i=0; i<DUZ; i++) niz[i] ^= b2.niz[i];
    niz[DUZ-1] &= MASKA;
    return *this;
  }

template <int D>
  Bitovi<D>& Bitovi<D>::operator>>=(int k) {       // b1 >>= k
    for (int i=0; i<D; i++) (*this)[i] = (i+k<D) ? (*this)[i+k] : 0;
    return *this;
  }

template <int D>
  Bitovi<D>& Bitovi<D>::operator<<=(int k) {       // b1 <<= k
    for (int i=D-1; i>=0; i--) (*this)[i] = (i-k>=0) ? (*this)[i-k] : 0;
    return *this;
  }

template <int D>
  inline Bitovi<D> operator& (CB& b1, CB& b2)      // b1 &  b2
    { return Bitovi<D> (b1) &= b2; }

template <int D>
  inline Bitovi<D> operator| (CB& b1, CB& b2)      // b1 |  b2
    { return Bitovi<D> (b1) |= b2; }

template <int D>
  inline Bitovi<D> operator^ (CB& b1, CB& b2)      // b1 ^  b2
    { return Bitovi<D> (b1) ^= b2; }

template <int D>
  inline Bitovi<D> operator>> (CB& b1, int k)      // b1 >> k
    { return Bitovi<D> (b1) >>= k; }

template <int D>
  inline Bitovi<D> operator<< (CB& b1, int k)      // b1 << k
    { return Bitovi<D> (b1) <<= k; }

template <int D>
  bool Bitovi<D>::operator== (CB& b2) const {      // b1 == b2
    for (int i=0; i<DUZ; i++)
      if (niz[i] != b2.niz[i]) return false;
    return true;
  } 

template <int D>
  bool Bitovi<D>::operator<  (CB& b2) const {      // b1 <  b2
    int i = DUZ-1;
    while (i>0 && niz[i]==b2.niz[i]) i--;
    return niz[i] < b2.niz[i];
  }

template <int D>
  bool Bitovi<D>::operator>  (CB& b2) const {      // b1 >  b2
    int i =DUZ-1;
    while (i>0 && niz[i]==b2.niz[i]) i--;
    return niz[i] > b2.niz[i];
  }

template <int D>
  inline bool   operator!= (CB& b1, CB& b2)        // b1 != b2
    { return ! (b1 == b2); }

template <int D>
  inline bool   operator<= (CB& b1, CB& b2)        // b1 <= b2
    { return ! (b1 >  b2); }

template <int D>
  inline bool   operator>= (CB& b1, CB& b2)        // b1 >= b2
    { return ! (b1 <  b2); }

template <int D>
  istream& operator>> (istream& d, Bitovi<D>& b) { // Citanje.
    b = 0;
    char txt[2*N]; d >> txt;
    // Uzimaju se u obzir najvise poslednjih D znakova.
    for (int i=0, j=strlen(txt)-1; j>=0 && i<D; b[i++]=txt[j--]!='0');
    return d;
  }

template <int D>
  ostream& operator<< (ostream& d, CB& b) {        // Pisanje.
    for (int i=D-1; i>=0; d << b[i--]);
    return d;
  }
  
#endif
