// oper.h - Klase operatora.

#ifndef _oper_h_
#define _oper_h_

#include "izraz.h"
#include <cmath>
using namespace std;

namespace Izrazi {
  class Oper: public Izraz { // APSTRAKTNA KLASA BINARNIH OPERATORA.
  protected:
    Izraz *a, *b;                           // Operandi.
    void pisi (ostream& d, char op) const   // Pisanje.
      { d << '(' << *a << op << *b << ')'; }
  private:
    void kopiraj (const Oper& op)           // Kopiranje u objekat.
      { a = op.a->kopija (); b = op.b->kopija (); }
    void brisi () { delete a; delete b; }   // Oslobadjanje memorije.
  public:
    Oper (const Izraz& x, const Izraz& y)   // Inicijalizacija izrazima.
      { a = x.kopija (); b = y.kopija (); }
    Oper (const Oper& op) { kopiraj (op); } // Konstruktor kopije.
    ~Oper () { brisi (); }                  // Destruktor.
    Oper& operator= (const Oper& y) {       // Dodela vrednosti.
      if (this != &y) { brisi (); kopiraj (y); }
      return *this;
    }
  }; // class Oper

  class Zbir: public Oper {                               // KLASA ZBIROVA.
    void pisi (ostream& d) const { Oper::pisi (d, '+'); } // Pisanje.
  public:
    Zbir (const Izraz& x, const Izraz& y): Oper (x, y) {} // Stvaranje.
    double operator+() const { return +*a + +*b; }        // Vrednost.
    Zbir* kopija () const { return new Zbir (*this); }    // Din. kopija.
  }; // class Zbir

  class Razl: public Oper {                               // KLASA RAZLIKA.
    void pisi (ostream& d) const { Oper::pisi (d, '-'); } // Pisanje.
  public:
    Razl (const Izraz& x, const Izraz& y): Oper (x, y) {} // Stvaranje.
    double operator+() const { return +*a - +*b; }        // Vrednost.
    Razl* kopija () const { return new Razl (*this); }    // Din. kopija.
  }; // class Razl

  class Proizv: public Oper {                             // KL. PROIZVODA.
    void pisi (ostream& d) const { Oper::pisi (d, '*'); } // Pisanje.
  public:
    Proizv (const Izraz& x, const Izraz& y): Oper (x,y) {}// Stvaranje.
    double operator+() const { return +*a * +*b; }        // Vrednost.
    Proizv* kopija () const { return new Proizv (*this); }// Din. kopjia.
  }; // class Proizv

  class Kolic: public Oper {                              // KL. KOLICNIKA.
    void pisi (ostream& d) const { Oper::pisi (d, '/'); } // Pisanje.
  public:
    Kolic (const Izraz& x, const Izraz& y): Oper (x, y) {}// Stvaranje.
    double operator+() const { return +*a / +*b; }        // Vrednost.
    Kolic* kopija () const { return new Kolic (*this); }  // Din. kopija.
  }; // class Kolic

  class Step: public Oper {                               // KLASA STEPENA.
    void pisi (ostream& d) const { Oper::pisi (d, '^'); } // Pisanje.
  public:
    Step (const Izraz& x, const Izraz& y): Oper (x, y) {} // Stvaranje.
    double operator+() const { return pow (+*a, +*b); }   // Vrednost.
    Step* kopija () const { return new Step (*this); }    // Din. kopija.
  }; // class Step

  // POMOCNE FUNKCIJE ZA OMOGUCAVANJE PISANJA CITLJIVIH IZRAZA.
  Zbir   operator+  (const Izraz& a, const Izraz& b)      // a +  b
    { return Zbir   (a, b); }
  Razl   operator-  (const Izraz& a, const Izraz& b)      // a -  b
    { return Razl   (a, b); }
  Proizv operator*  (const Izraz& a, const Izraz& b)      // a *  b
    { return Proizv (a, b); }
  Kolic  operator/  (const Izraz& a, const Izraz& b)      // a /  b
    { return Kolic  (a, b); }
  Step   operator^  (const Izraz& a, const Izraz& b)      // a ^  b
    { return Step   (a, b); }
  bool   operator== (const Izraz& a, const Izraz& b)      // a == b
    { return +a == +b; }
  bool   operator!= (const Izraz& a, const Izraz& b)      // a != b
    { return +a != +b; }
  bool   operator<  (const Izraz& a, const Izraz& b)      // a <  b
    { return +a <  +b; }
  bool   operator<= (const Izraz& a, const Izraz& b)      // a <= b
    { return +a <= +b; }
  bool   operator>  (const Izraz& a, const Izraz& b)      // a >  b
    { return +a >  +b; }
  bool   operator>= (const Izraz& a, const Izraz& b)      // a >= b
    { return +a >= +b; }

} // namespace Izrazi;

#endif